/**
 * Copyright 2020 Valentin Draganescu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { StatusCode } from "./status-code";
import { ApiError } from "./api-error";
import { Envelope } from "../response/envelope";
export declare class Response<ResponseBody extends Envelope = Envelope> {
    statusCode: StatusCode;
    errors?: ApiError[];
    body?: string;
    headers: {
        [key: string]: string;
    };
    constructor(statusCode?: StatusCode);
    setBody: (body: ResponseBody) => Response<ResponseBody>;
    getBody: () => ResponseBody | null;
    addHeader: (key: string, value: string) => void;
    addHeaders: (headers: {
        [key: string]: string;
    }) => void;
    setHeaders: (headers: {
        [key: string]: string;
    }) => void;
    toJSON(): {
        statusCode: StatusCode;
        headers: {
            [key: string]: string;
        };
        body: string | undefined;
    };
}
